@@ -285,6 +285,24 @@ int test_header_parsing(void)
       su_free(home, (void *)p), p = NULL;
     }
 
+    master = ";0";
+
+    for (i = 1; i < 256; i++) {
+      master = su_sprintf(home, "%s; %u", master, i); TEST_1(master);
+      list = end = su_strdup(home, master);
+      TEST_1(msg_params_d(NULL, &end, &p) >= 0);
+      TEST_S(end, "");
+      TEST_1(p);
+      for (j = 0; j <= i; j++) {
+	char number[10];
+	snprintf(number, sizeof number, "%u", j);
+	TEST_S(p[j], number);
+      }
+      TEST_1(p[i + 1] == NULL);
+      su_free(home, list);
+      su_free(NULL, (void *)p), p = NULL;
+    }
+
     su_home_deinit(home);
   }
 
@@ -478,6 +496,22 @@ int test_header_parsing(void)
     su_home_deinit(home);
   }
 
+  {
+    char b[8];
+    TEST(msg_unquoted_e(NULL, 0, "\"\""), 6);
+    TEST(msg_unquoted_e(b, 0, "\"\""), 6);
+    TEST(msg_unquoted_e(b, 4, "\"\""), 6);
+    TEST(msg_unquoted_e(b, 6, "\"\""), 6);
+    TEST(memcmp(b, "\"\\\"\\\"\"", 6), 0);
+    TEST(msg_unquoted_e(b, 4, "\""), 4);
+    memset(b, 0, sizeof b);
+    TEST(msg_unquoted_e(b, 1, "\"kuik"), 8);
+    TEST(memcmp(b, "\"\0", 2), 0);
+    TEST(msg_unquoted_e(b, 3, "\"kuik"), 8);
+    TEST(memcmp(b, "\"\\\"\0", 4), 0);
+    TEST(msg_unquoted_e(b, 7, "\"kuik"), 8);
+    TEST(memcmp(b, "\"\\\"kuik\0", 8), 0);
+  }
 
   END();
 }
@@ -572,7 +606,7 @@ int test_msg_parsing(void)
   msg_status_t *status;
   msg_content_location_t *location;
   msg_content_language_t *language;
-  msg_accept_language_t *se;
+  msg_accept_language_t *en, *se;
   msg_separator_t *separator;
   msg_payload_t *payload;
 
@@ -580,7 +614,9 @@ int test_msg_parsing(void)
 
   msg = read_msg("GET a-life HTTP/1.1" CRLF
 		 "Content-Length: 6" CRLF
+		 "Accept-Encoding: bzip2" CRLF
 		 "Accept-Language: en;q=0.8, fi, se ; q = 0.6" CRLF
+		 "Accept-Encoding: gzip" CRLF
 		 "Foo: bar" CRLF
 		 CRLF
 		 "test" CRLF);
@@ -637,6 +673,14 @@ int test_msg_parsing(void)
     TEST(vi, 0);
   }
 
+  /* Test msg_fragment_clear_chain() */
+  en = tst->msg_accept_language;
+  TEST_1(en->aa_common->h_data != NULL);
+  msg_fragment_clear_chain((msg_header_t *)en->aa_next->aa_next);
+  TEST_1(en->aa_common->h_data == NULL);
+  TEST_1(en->aa_next->aa_common->h_data == NULL);
+  TEST_1(en->aa_next->aa_next->aa_common->h_data == NULL);
+
   msg_destroy(msg);
 
   /* Bug #2624: */
@@ -722,6 +766,8 @@ int test_msg_parsing(void)
     TEST(msg_serialize(msg, (msg_pub_t *)tst), 0);
   }
 
+  msg_destroy(msg);
+
   /* Bug #2429 */
   orig = read_msg("GET a-life HTTP/1.1" CRLF
 		 "Foo: bar" CRLF
@@ -734,6 +780,7 @@ int test_msg_parsing(void)
   TEST_1(otst);
 
   msg = msg_copy(orig);
+  msg_destroy(orig);
   tst = msg_test_public(msg);
   TEST_1(tst);
 
@@ -1193,8 +1240,15 @@ int test_mime(void)
     CRLF			/* 13 */
 #define BODY3 "<html><body>part 3</body></html>" CRLF
     BODY3			/* 14 */
-    CRLF			/* 15 */
-    "--LaGqGt4BI6Ho--" CRLF;
+    CRLF "--LaGqGt4BI6Ho"	/* 15 */
+    "c: text/html" CRLF		/* 16 */
+    "l: 9" CRLF			/* 17 */
+    "e: identity" CRLF		/* 18 */
+    CRLF			/* 19 */
+#define BODY4 "<html/>" CRLF
+    BODY4			  /* 20 */
+    CRLF "--LaGqGt4BI6Ho--"	  /* 21 */
+    CRLF;
 
   BEGIN();
 
@@ -1275,6 +1329,18 @@ int test_mime(void)
   TEST_SIZE(strlen(BODY3), pl->pl_len);
   TEST(memcmp(pl->pl_data, BODY3, pl->pl_len), 0);
 
+  TEST_1(mp = mp->mp_next);
+
+  TEST_1(mp->mp_data);
+  TEST_M(mp->mp_data, CRLF "--" "LaGqGt4BI6Ho" CRLF, mp->mp_len);
+
+  TEST_1(mp->mp_content_encoding);
+  TEST_1(mp->mp_content_type);
+
+  TEST_1(pl = mp->mp_payload); TEST_1(pl->pl_data);
+  TEST_SIZE(strlen(BODY4), pl->pl_len);
+  TEST(memcmp(pl->pl_data, BODY4, pl->pl_len), 0);
+
   mpX = mp;
 
   TEST_1(!(mp = mp->mp_next));
@@ -1293,7 +1359,7 @@ int test_mime(void)
     h->sh_succ = NULL;
   }
 
-  TEST(n, 15);
+  TEST(n, 21);
 
   head = NULL;
   TEST_1(h = msg_multipart_serialize(&head, mp0));
@@ -1304,7 +1370,7 @@ int test_mime(void)
     h_succ = h->sh_succ;
   }
 
-  TEST(n, 15);
+  TEST(n, 21);
 
   /* Add a new part to multipart */
   mpnew = su_zalloc(home, sizeof(*mpnew)); TEST_1(mpnew);
@@ -1324,7 +1390,7 @@ int test_mime(void)
     TEST_1(h != removed);
   }
 
-  TEST(n, 19);
+  TEST(n, 21 + 4);
 
 #define remove(h) \
   (((*((msg_header_t*)(h))->sh_prev = ((msg_header_t*)(h))->sh_succ) ? \
@@ -1336,7 +1402,7 @@ int test_mime(void)
   remove(mp0->mp_separator);
   remove(mp0->mp_next->mp_payload);
   remove(mp0->mp_next->mp_next->mp_content_type);
-  remove(mp0->mp_next->mp_next->mp_next->mp_close_delim);
+  remove(mp0->mp_next->mp_next->mp_next->mp_next->mp_close_delim);
 
   TEST_1(!msg_chain_errors((msg_header_t *)mp0));
 
@@ -1352,7 +1418,7 @@ int test_mime(void)
     TEST_1(h != removed);
   }
 
-  TEST(n, 19);
+  TEST(n, 21 + 4);
 
   /* Add an recursive multipart */
   mpnew = su_zalloc(home, sizeof(*mpnew)); TEST_1(mpnew);
@@ -1371,7 +1437,7 @@ int test_mime(void)
   for (h = (msg_header_t *)mp0, n = 0; h; h = h_succ, n++)
     h_succ = h->sh_succ;
 
-  TEST(n, 24);
+  TEST(n, 21 + 9);
 
   su_home_check(home);
   su_home_zap(home);
