@@ -59,6 +59,7 @@ typedef struct agent_t agent_t;
 #include <sofia-sip/msg_mclass.h>
 #include <sofia-sip/sofia_features.h>
 #include <sofia-sip/hostdomain.h>
+#include <sofia-sip/su_string.h>
 
 #include <stddef.h>
 #include <stdlib.h>
@@ -67,7 +68,6 @@ typedef struct agent_t agent_t;
 #include <stdio.h>
 #include <assert.h>
 #include <time.h>
-#include "sofia-sip/string0.h"
 
 extern su_log_t nta_log[];
 extern su_log_t tport_log[];
@@ -489,7 +489,6 @@ static int api_test_destroy(agent_t *ag)
   TEST_VOID(su_home_deinit(home));
 
   END();
-
 }
 
 
@@ -763,14 +762,14 @@ int api_test_tport(agent_t *ag)
 
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url, TAG_END()) == 0);
   TEST_1(v = nta_agent_via(agent)); TEST_1(!v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_tcp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_tcp), 0);
   TEST_1(m = nta_agent_contact(agent));
   TEST_S(m->m_url->url_params, "transport=tcp");
 
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url,
 			     TPTAG_SERVER(0), TAG_END()) == 0);
   TEST_1(v = nta_agent_public_via(agent)); TEST_1(!v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_tcp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_tcp), 0);
   TEST_1(host_has_domain_invalid(v->v_host));
   TEST_1(m = nta_agent_contact(agent));
   TEST_S(m->m_url->url_params, "transport=tcp");
@@ -778,14 +777,14 @@ int api_test_tport(agent_t *ag)
   url->url_params = "transport=udp";
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url, TAG_END()) == 0);
   TEST_1(v = nta_agent_via(agent)); TEST_1(v = v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_udp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_udp), 0);
 
   TEST_VOID(nta_agent_destroy(agent));
 
   TEST_1(agent = nta_agent_create(ag->ag_root, NONE, NULL, NULL, TAG_END()));
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url, TAG_END()) == 0);
   TEST_1(v = nta_agent_via(agent)); TEST_1(!v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_udp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_udp), 0);
   TEST_1(m = nta_agent_contact(agent));
   TEST_S(m->m_url->url_params, "transport=udp");
   TEST_VOID(nta_agent_destroy(agent));
@@ -795,9 +794,9 @@ int api_test_tport(agent_t *ag)
   TEST_1(agent = nta_agent_create(ag->ag_root, NONE, NULL, NULL, TAG_END()));
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url, TAG_END()) == 0);
   TEST_1(v = nta_agent_via(agent));
-  TEST(strcasecmp(v->v_protocol, sip_transport_tcp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_tcp), 0);
   TEST_1(v = v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_udp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_udp), 0);
   TEST_1(m = nta_agent_contact(agent));
   TEST_1(!m->m_url->url_params);
   TEST_VOID(nta_agent_destroy(agent));
@@ -807,9 +806,9 @@ int api_test_tport(agent_t *ag)
   TEST_1(agent = nta_agent_create(ag->ag_root, NONE, NULL, NULL, TAG_END()));
   TEST_1(nta_agent_add_tport(agent, (url_string_t *)url, TAG_END()) == 0);
   TEST_1(v = nta_agent_via(agent));
-  TEST(strcasecmp(v->v_protocol, sip_transport_udp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_udp), 0);
   TEST_1(v = v->v_next);
-  TEST(strcasecmp(v->v_protocol, sip_transport_tcp), 0);
+  TEST(!su_casematch(v->v_protocol, sip_transport_tcp), 0);
   TEST_1(m = nta_agent_contact(agent));
   TEST_1(!m->m_url->url_params);
   TEST_VOID(nta_agent_destroy(agent));
@@ -865,6 +864,90 @@ static int api_test_dialogs(agent_t *ag)
 }
 
 
+/* Test that NULL host and/or port fields of user supplied Via header are
+   filled in automatically */
+int api_test_user_via_fillin(agent_t *ag)
+{
+  su_home_t home[1];
+  su_root_t *root;
+  nta_agent_t *nta;
+  nta_leg_t *leg;
+  nta_outgoing_t *orq0, *orq1;
+  msg_t *msg0, *msg1;
+  sip_t *sip0, *sip1;
+  sip_via_t *via0, *via1;
+  sip_via_t via[1];
+  static char *via_params[] = { "param1=value1", "param2=value2" };
+  size_t i;
+
+  BEGIN();
+
+  memset(home, 0, sizeof home);
+  su_home_init(home);
+
+  TEST_1(root = su_root_create(NULL));
+
+  TEST_1(nta = nta_agent_create(root,
+  			  (url_string_t *)"sip:*:*",
+  			  NULL,
+  			  NULL,
+  			  TAG_END()));
+  TEST_1(leg = nta_leg_tcreate(nta, NULL, NULL,
+  			 NTATAG_NO_DIALOG(1),
+  			 TAG_END()));
+
+  /* This creates a delayed response message */
+  orq0 = nta_outgoing_tcreate(leg, outgoing_callback, ag, NULL,
+  		       SIP_METHOD_MESSAGE,
+  		       URL_STRING_MAKE("sip:foo.bar;transport=none"),
+  		       SIPTAG_FROM_STR("<sip:bar.foo>"),
+  		       SIPTAG_TO_STR("<sip:foo.bar>"),
+  		       TAG_END());
+  TEST_1(orq0);
+  TEST_1(msg0 = nta_outgoing_getrequest(orq0));
+  TEST_1(sip0 = sip_object(msg0));
+  TEST_1(via0 = sip0->sip_via);
+
+  /* create user Via template to be filled in by NTA */
+  sip_via_init(via);
+  via->v_protocol = "*";
+  for (i = 0; i < sizeof(via_params) / sizeof(via_params[0]); i++)
+    sip_via_add_param(home, via, via_params[i]); /* add param to the template */
+
+  /* This creates a delayed response message */
+  orq1 = nta_outgoing_tcreate(leg, outgoing_callback, ag, NULL,
+			      SIP_METHOD_MESSAGE,
+			      URL_STRING_MAKE("sip:foo.bar;transport=none"),
+			      SIPTAG_FROM_STR("<sip:bar.foo>"),
+			      SIPTAG_TO_STR("<sip:foo.bar>"),
+			      NTATAG_USER_VIA(1),
+			      SIPTAG_VIA(via),
+			      TAG_END());
+  TEST_1(orq1);
+  TEST_1(msg1 = nta_outgoing_getrequest(orq1));
+  TEST_1(sip1 = sip_object(msg1));
+  TEST_1(via1 = sip1->sip_via);
+
+  /* check that template has been filled correctly */
+  TEST_S(via0->v_protocol, via1->v_protocol);
+  TEST_S(via0->v_host, via1->v_host);
+  TEST_S(via0->v_port, via1->v_port);
+  /* check that the parameter has been preserved */
+  for (i = 0; i < sizeof(via_params)/sizeof(via_params[0]); i++)
+    TEST_S(via1->v_params[i], via_params[i]);
+
+  TEST_VOID(nta_outgoing_destroy(orq0));
+  TEST_VOID(nta_outgoing_destroy(orq1));
+  TEST_VOID(nta_leg_destroy(leg));
+  TEST_VOID(nta_agent_destroy(nta));
+
+  TEST_VOID(su_root_destroy(root));
+  TEST_VOID(su_home_deinit(home));
+
+  END();
+}
+
+
 int outgoing_default(agent_t *ag,
 		     nta_outgoing_t *orq,
 		     sip_t const *sip)
@@ -1426,6 +1509,7 @@ int main(int argc, char *argv[])
     retval |= api_test_tport(ag); SINGLE_FAILURE_CHECK();
     retval |= api_test_dialogs(ag); SINGLE_FAILURE_CHECK();
     retval |= api_test_default(ag); SINGLE_FAILURE_CHECK();
+    retval |= api_test_user_via_fillin(ag); SINGLE_FAILURE_CHECK();
   }
   retval |= api_test_deinit(ag); fflush(stdout);
 
