@@ -71,7 +71,9 @@
 
 SOFIA_BEGIN_DECLS
 
+#ifdef DOXYGEN
 extern char const STUN_DEBUG[]; /* dummy declaration for Doxygen */
+#endif
 
 /* XXX -- mela: note that this are 100 times too small */
 #if 1
@@ -96,8 +98,10 @@ int stun_make_sharedsecret_req(stun_msg_t *msg);
 int stun_send_message(su_socket_t s, su_sockaddr_t *srvr,
 		      stun_msg_t *msg, stun_buffer_t *pwd);
 
+void stun_free_message_data(stun_msg_t *msg);
+
 int stun_make_binding_req(stun_handle_t *se, stun_request_t *req,
-			  stun_msg_t *msg, 
+			  stun_msg_t *msg,
 			  tag_type_t, tag_value_t, ...);
 int stun_process_response(stun_msg_t *msg);
 
