@@ -101,6 +101,8 @@ SU_DLL int su_home_threadsafe(su_home_t *home);
 
 SU_DLL int su_home_has_parent(su_home_t const *home);
 
+SU_DLL su_home_t *su_home_parent(su_home_t const *home);
+
 SU_DLL void su_home_check(su_home_t const *home);
 
 SU_DLL int su_home_check_alloc(su_home_t const *home, void const *data);
@@ -131,7 +133,7 @@ SU_DLL char *su_strndup(su_home_t *home, char const *s, isize_t n)
      __attribute__((__malloc__));
 
 SU_DLL char *su_strcat_all(su_home_t *home, ...)
-     __attribute__((__malloc__));
+     __attribute__((__malloc__, __sentinel__ (0)));
 
 SU_DLL char *su_sprintf(su_home_t *home, char const *fmt, ...)
      __attribute__ ((__malloc__, __format__ (printf, 2, 3)));
