@@ -116,8 +116,9 @@ SOFIAPUBFUN int su_randint(int lb, int ub);
  */
 SOFIAPUBFUN void *su_randmem(void *mem, size_t siz);
 
-/** Generate a random 32-bit integer. */
-SOFIAPUBFUN uint32_t su_random();
+SOFIAPUBFUN uint32_t su_random(void);
+
+SOFIAPUBFUN uint64_t su_random64(void);
 
 SOFIA_END_DECLS
 
