@@ -86,6 +86,8 @@ struct su_root_s;
  *    - su_root_run() [Do not call from cloned task]
  *    - su_root_break() [Do not call from cloned task]
  *    - su_root_step() [Do not call from cloned task]
+ *    - su_root_get_max_defer()
+ *    - su_root_set_max_defer()
  *    - su_root_task()
  *
  * New tasks can be created via su_clone_start() function.
@@ -115,10 +117,19 @@ struct su_root_s;
  * Contains hint of number of sockets supported by su_root_t */
 int su_root_size_hint = 64;
 
+/* ====================================================================== */
+
+_su_task_r su_task_new(su_task_r task, su_root_t *root, su_port_t *port);
+int su_task_attach(su_task_r self, su_root_t *root);
+int su_task_detach(su_task_r self);
+
+int su_timer_reset_all(su_timer_t **t0, su_task_r);
+
 /* =========================================================================
  * Tasks
  */
 
+/** NULL task. */
 su_task_r const su_task_null = SU_TASK_R_INIT;
 
 #define SU_TASK_ZAP(t, f) \
@@ -251,13 +262,13 @@ int su_task_cmp(su_task_r const a, su_task_r const b)
  *
  * @retval true (nonzero) if task is not stopped,
  * @retval zero if it is null or stopped.
+ *
+ * @note A task sharing thread with another task is considered stopped when
+ * ever the the main task is stopped.
  */
 int su_task_is_running(su_task_r const task)
 {
-  return
-    task &&
-    task->sut_port &&
-    task->sut_root;
+  return task && task->sut_root && su_port_is_running(task->sut_port);
 }
 
 /** @internal
@@ -310,14 +321,43 @@ int su_task_detach(su_task_r self)
  *
  * @param task task handle
  *
- * @return A timer list of the task. If there are no timers, it returns
- * NULL.
+ * @return A timer list of the task.
  */
 su_timer_queue_t *su_task_timers(su_task_r const task)
 {
   return task->sut_port ? su_port_timers(task->sut_port) : NULL;
 }
 
+/**Return the queue for deferrable timers associated with given task.
+ *
+ * @param task task handle
+ *
+ * @return A timer list of the task.
+ *
+ * @NEW_1_12_11
+ */
+su_timer_queue_t *su_task_deferrable(su_task_r const task)
+{
+  return task ? su_port_deferrable(task->sut_port) : NULL;
+}
+
+/** Wakeup a task.
+ *
+ * Wake up a task. This function is mainly useful when using deferrable
+ * timers executed upon wakeup.
+ *
+ * @param task task handle
+ *
+ * @retval 0 if succesful
+ * @retval -1 upon an error
+ *
+ * @NEW_1_12_11
+ */
+int su_task_wakeup(su_task_r const task)
+{
+  return task ? su_port_wakeup(task->sut_port) : -1;
+}
+
 /** Execute the @a function by @a task thread.
  *
  * @retval 0 if successful
@@ -348,12 +388,6 @@ int su_task_execute(su_task_r const task,
   }
 }
 
-_su_task_r su_task_new(su_task_r task, su_root_t *root, su_port_t *port);
-int su_task_attach(su_task_r self, su_root_t *root);
-int su_task_detach(su_task_r self);
-
-int su_timer_reset_all(su_timer_t **t0, su_task_r);
-
 /* Note that is *not* necessary same as su_root_t,
  * as su_root_t can be extended */
 
@@ -375,6 +409,49 @@ su_root_t *su_root_create(su_root_magic_t *magic)
   return su_root_create_with_port(magic, su_port_create());
 }
 
+/* Initializer used by su_root_clone() */
+static int
+su_root_clone_initializer(su_root_t *root,
+			  su_root_magic_t *magic)
+{
+  *(su_root_t **)magic = root;
+  return 0;
+}
+
+/** Create a a new root object sharing port/thread with existing one.
+ *
+ * Allocate and initialize the instance of su_root_t.
+ *
+ * @param self      pointer to a root object.
+ * @param magic     pointer to user data
+ *
+ * @return A pointer to allocated su_root_t instance, NULL on error.
+ *
+ * @NEW_1_12_11
+ */
+su_root_t *
+su_root_clone(su_root_t *self, su_root_magic_t *magic)
+{
+  int threading = 0, error;
+  su_clone_r clone;
+  su_root_t *cloned = NULL;
+
+  if (self == NULL)
+    return NULL;
+
+  threading = self->sur_threading, self->sur_threading = 0;
+  error = su_clone_start(self, clone,
+			 (void *)&cloned, su_root_clone_initializer, NULL);
+  self->sur_threading = threading;
+
+  if (error)
+    return NULL;
+
+  su_clone_forget(clone);	/* destroyed with su_root_destroy() */
+  su_root_set_magic(cloned, magic);
+  return cloned;
+}
+
 /**@internal
  *
  * Create a reactor object using given message port.
@@ -445,6 +522,10 @@ void su_root_destroy(su_root_t *self)
   unregistered = su_port_unregister_all(port, self);
   reset = su_timer_reset_all(su_task_timers(self->sur_task), self->sur_task);
 
+  if (su_task_deferrable(self->sur_task))
+    reset += su_timer_reset_all(su_task_deferrable(self->sur_task),
+				self->sur_task);
+
   if (unregistered || reset)
     SU_DEBUG_1(("su_root_destroy: "
 		"%u registered waits, %u timers\n",
@@ -599,6 +680,56 @@ int su_root_unregister(su_root_t *self,
   return su_port_unregister(self->sur_port, self, wait, callback, arg);
 }
 
+/** Set maximum defer time.
+ *
+ * The deferrable timers can be deferred until the task is otherwise
+ * activated, however, they are deferred no longer than the maximum defer
+ * time. The maximum defer time determines also the maximum time during
+ * which task waits for events while running.  The maximum defer time is 15
+ * seconds by default.
+ *
+ * Cloned tasks inherit the maximum defer time.
+ *
+ * @param self pointer to root object
+ * @param max_defer maximum defer time in milliseconds
+ *
+ * @retval 0 when successful
+ * @retval -1 upon an error
+ *
+ * @sa su_timer_deferrable()
+ *
+ * @NEW_1_12_11
+ */
+int su_root_set_max_defer(su_root_t *self, su_duration_t max_defer)
+{
+  if (!self)
+    return -1;
+
+  return su_port_max_defer(self->sur_port, &max_defer, &max_defer);
+}
+
+/** Get maximum defer time.
+ *
+ * The deferrable timers can be deferred until the task is otherwise
+ * activated, however, they are deferred no longer than the maximum defer
+ * time. The maximum defer time is 15 seconds by default.
+ *
+ * @param self pointer to root object
+ *
+ * @return Maximum defer time
+ *
+ * @NEW_1_12_7
+ */
+su_duration_t su_root_get_max_defer(su_root_t const *self)
+{
+  su_duration_t max_defer = SU_WAIT_MAX;
+
+  if (self != NULL)
+    su_port_max_defer(self->sur_port, &max_defer, NULL);
+
+  return max_defer;
+}
+
 /** Remove a su_wait_t registration.
  *
  *  The function su_root_deregister() deregisters a su_wait_t object. The
