@@ -139,7 +139,7 @@ su_port_vtable_t const su_select_port_vtable[1] =
       su_base_port_incref,
       su_select_port_decref,
       su_base_port_gsource,
-      su_socket_port_send,
+      su_base_port_send,
       su_select_port_register,
       su_select_port_unregister,
       su_select_port_deregister,
@@ -160,6 +160,12 @@ su_port_vtable_t const su_select_port_vtable[1] =
       su_base_port_start_shared,
       su_pthread_port_wait,
       su_pthread_port_execute,
+      su_base_port_deferrable,
+      su_base_port_max_defer,
+      su_socket_port_wakeup,
+      su_base_port_is_running,
+      su_base_port_stamp64,
+      su_base_port_stamp64_offset,
     }};
 
 static char const *su_select_port_name(su_port_t const *self)
