@@ -118,7 +118,7 @@ su_port_vtable_t const su_wsevent_port_vtable[1] =
       su_base_port_incref,
       su_wsevent_port_decref,
       su_base_port_gsource,
-      su_socket_port_send,
+      su_base_port_send,
       su_wsevent_port_register,
       su_wsevent_port_unregister,
       su_wsevent_port_deregister,
@@ -139,6 +139,10 @@ su_port_vtable_t const su_wsevent_port_vtable[1] =
       su_base_port_start_shared,
       su_pthread_port_wait,
       su_pthread_port_execute,
+      su_base_port_deferrable,
+      su_base_port_max_defer,
+      su_socket_port_wakeup,
+      su_base_port_is_running,
     }};
 
 static char const *su_wsevent_port_name(su_port_t const *self)
