@@ -52,6 +52,7 @@ typedef struct tp_test_s tp_test_t;
 
 #include <sofia-sip/su_wait.h>
 #include <sofia-sip/su_md5.h>
+#include <sofia-sip/su_localinfo.h>
 
 #include "tport_internal.h"	/* Get SU_DEBUG_*() */
 
@@ -472,6 +473,7 @@ static int init_test(tp_test_t *tt)
   tp_name_t const *tpn;
   tport_t *tp;
   unsigned idle;
+  int logging = -1;
 
   BEGIN();
 
@@ -552,23 +554,39 @@ static int init_test(tp_test_t *tt)
 			TPTAG_IDLE_REF(idle),
 			TAG_END()), 1);
 
+  /* Check that logging tag works */
+  TEST(tport_get_params(tt->tt_srv_tports,
+			TPTAG_LOG_REF(logging),
+			TAG_END()), 1);
+  TEST(tport_set_params(tt->tt_srv_tports,
+			TPTAG_LOG(logging),
+			TAG_END()), 1);
+
+
   for (tp = tport_primaries(tt->tt_srv_tports); tp; tp = tport_next(tp))
     TEST_S(tport_name(tp)->tpn_ident, "server");
 
   {
-    su_sockaddr_t su[1];
+    su_sockaddr_t *su;
     socklen_t sulen;
     int s;
     int i, before, after;
     char port[8];
 
     tp_name_t rname[1];
+    su_localinfo_t *li, hints[1] = {{ 0 }};
 
     *rname = *myname;
 
     /* Check that we cannot bind to an already used socket */
 
-    memset(su, 0, sulen = sizeof(su->su_sin));
+    /* Windows allows concurrent binding to a wildcard */
+    memset(hints, 0, sizeof hints);
+    if (strcmp(rname->tpn_host, "*"))
+      hints->li_canonname = (char *) rname->tpn_host;
+    TEST_1(su_getlocalinfo(hints, &li) == 0);
+    su = li->li_addr, sulen = li->li_addrlen;
+
     s = su_socket(su->su_family = AF_INET, SOCK_STREAM, 0); TEST_1(s != -1);
     TEST_1(bind(s, &su->su_sa, sulen) != -1);
     TEST_1(listen(s, 5) != -1);
@@ -589,6 +607,8 @@ static int init_test(tp_test_t *tt)
 
     after = count_tports(tt->tt_srv_tports);
 
+    su_freelocalinfo(li);
+
     /* Check that no new primary transports has been added by failed call */
     TEST(before, after);
 
@@ -1298,6 +1318,7 @@ static int tls_test(tp_test_t *tt)
   TEST_1(pending_client_close > 0);
   tp = tt->tt_rtport;
   pending_server_close = tport_pend(tp, NULL, server_closed_callback, NULL);
+
   TEST_1(pending_server_close > 0);
 
   /* Send a largish message */
