@@ -129,22 +129,27 @@ static int tport_thread_send(tport_t *tp,
 
 tport_vtable_t const tport_threadpool_vtable =
 {
-  "udp", tport_type_local,
-  sizeof (tport_threadpool_t),
-  tport_threadpool_init_primary,
-  tport_threadpool_deinit_primary,
-  NULL,
-  NULL,
-  0,				/* No secondary transports! */
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  NULL,
-  tport_recv_dgram,
-  tport_send_dgram,
-  NULL,
-  tport_thread_send
+  /* vtp_name 		     */ "udp",
+  /* vtp_public              */ tport_type_local,
+  /* vtp_pri_size            */ sizeof (tport_threadpool_t),
+  /* vtp_init_primary        */ tport_threadpool_init_primary,
+  /* vtp_deinit_primary      */ tport_threadpool_deinit_primary,
+  /* vtp_wakeup_pri          */ NULL,
+  /* vtp_connect             */ NULL,
+  /* vtp_secondary_size      */ 0, /* No secondary transports! */
+  /* vtp_init_secondary      */ NULL,
+  /* vtp_deinit_secondary    */ NULL,
+  /* vtp_shutdown            */ NULL,
+  /* vtp_set_events          */ NULL,
+  /* vtp_wakeup              */ NULL,
+  /* vtp_recv                */ tport_recv_dgram,
+  /* vtp_send                */ tport_send_dgram,
+  /* vtp_deliver             */ NULL,
+  /* vtp_prepare             */ tport_thread_send,
+  /* vtp_keepalive           */ NULL,
+  /* vtp_stun_response       */ NULL,
+  /* vtp_next_secondary_timer*/ NULL,
+  /* vtp_secondary_timer     */ NULL,
 };
 
 static int thrp_udp_init(su_root_t *, threadpool_t *);
@@ -657,7 +662,7 @@ int tport_thread_send(tport_t *tp,
   SU_DEBUG_9(("tport_thread_send()\n"));
 
   if (thrp->thrp_killing)
-    return (su_seterrno(ECHILD)), -1;
+    return (su_seterrno(ECHILD));
 
   qlen = totalqlen = thrp->thrp_s_sent - thrp->thrp_s_recv;
 
@@ -690,7 +695,7 @@ int tport_thread_send(tport_t *tp,
   tpd->tpd_thrp = thrp;
   tpd->tpd_when = su_now();
   tpd->tpd_mtu = mtu;
-  tpd->tpd_msg = msg_ref_create(msg);
+  tpd->tpd_msg = msg_ref(msg);
 
 #if HAVE_SIGCOMP
   tpd->tpd_cc = cc;
@@ -703,7 +708,7 @@ int tport_thread_send(tport_t *tp,
     return 0;
   }
 
-  msg_ref_destroy(msg);
+  msg_unref(msg);
   return -1;
 }
 
@@ -808,5 +813,6 @@ void thrp_udp_send_report(su_root_magic_t *magic,
   if (tpd->tpd_errorcode)
     tport_error_report(tp, tpd->tpd_errorcode, msg_addr(tpd->tpd_msg));
 
-  msg_ref_destroy(tpd->tpd_msg);
+  msg_unref(tpd->tpd_msg);
+
 }
