@@ -39,6 +39,8 @@
 #include <sofia-sip/su_types.h>
 #endif
 
+#include "tport_internal.h"
+
 SOFIA_BEGIN_DECLS
 
 #define TLS_MAX_HOSTS (16)
@@ -48,12 +50,13 @@ typedef struct tls_s tls_t;
 extern char const tls_version[];
 
 typedef struct tls_issues_s {
-  int   verify_peer;    /* 0: no verify certificate, *
-                         * 1: if fail the TLS/SSL handshake is terminated. */
-  int   verify_depth;   /* if 0, then do nothing                      */
+  unsigned policy;      /* refer to tport_tag.h, tport_tls_verify_policy */
+  unsigned verify_depth;/* if 0, revert to default (2) */
+  unsigned verify_date; /* if 0, notBefore and notAfter dates are ignored */
   int   configured;	/* If non-zero, complain about certificate errors */
   char *cert;		/* CERT file name. File format is PEM         */
   char *key;		/* Private key file. PEM format               */
+  char *passphrase;     /* Passphrase for password protected private key */
   char *randFile;       /* Seed file for the PRNG (default: tls_seed.dat) */
   char *CAfile;		/* PEM file of CA's                           */
   char *CApath;		/* PEM file path of CA's		      */
@@ -64,9 +67,19 @@ typedef struct tls_issues_s {
 			 * used, it is 0. */
 } tls_issues_t;
 
+typedef struct tport_tls_s {
+  tport_t  tlstp_tp[1];
+  tls_t   *tlstp_context;
+  char    *tlstp_buffer;
+} tport_tls_t;
+
+typedef struct tport_tls_primary_s {
+  tport_primary_t tlspri_pri[1];
+  tls_t *tlspri_master;
+} tport_tls_primary_t;
+
 tls_t *tls_init_master(tls_issues_t *tls_issues);
-tls_t *tls_init_slave(tls_t *tls_master, int sock);
-tls_t *tls_init_client(tls_t *tls_master, int sock);
+tls_t *tls_init_secondary(tls_t *tls_master, int sock, int accept);
 void tls_free(tls_t *tls);
 int tls_get_socket(tls_t *tls);
 ssize_t tls_read(tls_t *tls);
@@ -74,11 +87,10 @@ void *tls_read_buffer(tls_t *tls, size_t N);
 int tls_want_read(tls_t *tls, int events);
 int tls_pending(tls_t const *tls);
 
+int tls_connect(su_root_magic_t *magic, su_wait_t *w, tport_t *self);
 ssize_t tls_write(tls_t *tls, void *buf, size_t size);
 int tls_want_write(tls_t *tls, int events);
 
-int tls_check_hosts(tls_t *tls, char const *hosts[TLS_MAX_HOSTS]);
-
 int tls_events(tls_t const *tls, int flags);
 
 SOFIA_END_DECLS
