@@ -72,7 +72,7 @@ int save_until_notified_and_responded(CONDITION_PARAMS)
 		   NUTAG_AUTH("Digest:\"test-proxy\":charlie:secret"),
 		   TAG_END());
     }
-    else if (status >= 300) 
+    else if (status >= 300)
       return 1;
     else if (status >= 200)
       ep->flags.bit1 = 1;
@@ -311,9 +311,11 @@ int test_events(struct context *ctx)
   if (print_headings)
     printf("TEST NUA-12.5: un-SUBSCRIBE\n");
 
+  memset(&a->flags, 0, sizeof a->flags);
+
   UNSUBSCRIBE(a, a_call, a_call->nh, TAG_END());
 
-  run_ab_until(ctx, -1, save_until_final_response,
+  run_ab_until(ctx, -1, save_until_notified_and_responded,
 	       -1, save_until_subscription);
 
   /* Client events:
@@ -330,15 +332,32 @@ int test_events(struct context *ctx)
     TEST_1(!sip->sip_subscription_state->ss_expires);
     TEST_1(tl_find(n_tags, nutag_substate));
     TEST(tl_find(n_tags, nutag_substate)->t_value, nua_substate_terminated);
+
     TEST_1(e = e->next);
-  }
-  TEST_E(e->data->e_event, nua_r_unsubscribe);
-  TEST_1(tl_find(e->data->e_tags, nutag_substate));
-  TEST(tl_find(e->data->e_tags, nutag_substate)->t_value,
+    TEST_E(e->data->e_event, nua_r_unsubscribe);
+    TEST_1(tl_find(e->data->e_tags, nutag_substate));
+    TEST(tl_find(e->data->e_tags, nutag_substate)->t_value,
        nua_substate_terminated);
-  /* Currently, NOTIFY is dropped after successful response to unsubscribe */
-  /* But we don't really care.. */
-  /* TEST_1(!e->next); */
+  }
+  else {
+    TEST_E(e->data->e_event, nua_r_unsubscribe);
+    TEST_1(tl_find(e->data->e_tags, nutag_substate));
+    /* NOTIFY is no more dropped after successful response to unsubscribe */
+    TEST(tl_find(e->data->e_tags, nutag_substate)->t_value,
+       nua_substate_active);
+
+    TEST_1(e = e->next);
+    TEST_E(e->data->e_event, nua_i_notify);
+    TEST_1(sip = sip_object(e->data->e_msg));
+    n_tags = e->data->e_tags;
+    TEST_1(sip->sip_event);
+    TEST_1(sip->sip_subscription_state);
+    TEST_S(sip->sip_subscription_state->ss_substate, "terminated");
+    TEST_1(!sip->sip_subscription_state->ss_expires);
+    TEST_1(tl_find(n_tags, nutag_substate));
+    TEST(tl_find(n_tags, nutag_substate)->t_value, nua_substate_terminated);
+  }
+  TEST_1(!e->next);
   free_events_in_list(ctx, a->events);
 
   /* Server events: nua_i_subscription with terminated status */
